# Governing R code to run analyses for 
# Measurement and Uncertainty Preserving Parametric (MUPPET) CFA on
# A simulated example

# Load the packages needed -----
library(dplyr)
library(blavaan)
future::plan("multisession")
options(mc.cores = parallel::detectCores())
library(R2jags)
library(MCMCvis)
library(coda)
library(parallel)
library(foreach)
library(doParallel)
library(MASS)
library(lavaan)
library(rstatix)
library(stringr)


# Load the functions -----
source("LR function.R")
source("Residual correlation matrix function.R")
source("ICDM matrix function.R")
source("MUPPET CFA function.R")


# Read in the data ------
raw.data <- as.data.frame(read.table("gen.data.1.dat", header = TRUE))


# Conduct MUPPET modeling with a covariate ------
MUPPET.CFA.covariate <- MUPPET.CFA.function(
  data=raw.data,
  indicators.names=c("x1", "x2", "x3", "x4"),
  covariates.names="w1",
  outcomes.names=NULL,
  measurement.model.priors=dpriors(
    lambda="normal(0,10)",    # measured variable loadings in N(mean, sd)
    theta="gamma(1,.8)[prec]"     # measured variable error precision
  ),
  n.chains = 2,
  n.warmup = 500,
  n.burnin = 0,
  n.iters.per.chain.after.warmup.and.burnin = 10000,
  obtain.standardized.cfa=TRUE,
  beta.o.prior.mean = 0,
  beta.o.prior.var = 100,
  psi.y.prior.alpha = 1,
  psi.y.prior.beta = 1,
  beta.c.prior.mean = 0,
  beta.c.prior.var = 100,
  n.iters.per.chain.total.structural = 51,
  save.summary.stats.from.MUPPET=TRUE,
  save.draws.from.MUPPET=FALSE,
  model.check=TRUE,
  save.post.pred.data=FALSE
)

# Examine summary statistics from model fitting, model-checking ------
MUPPET.CFA.covariate$summary.statistics.MUPPET
MUPPET.CFA.covariate$summary.statistics.PPMC


# Conduct MUPPET modeling with an outcome ------
MUPPET.CFA.outcome <- MUPPET.CFA.function(
  data=raw.data,
  indicators.names=c("x1", "x2", "x3", "x4"),
  covariates.names=NULL,
  outcomes.names="y1",
  measurement.model.priors=dpriors(
    lambda="normal(0,10)",    # measured variable loadings in N(mean, sd)
    theta="gamma(1,.8)[prec]"     # measured variable error precision
  ),
  n.chains = 2,
  n.warmup = 500,
  n.burnin = 0,
  n.iters.per.chain.after.warmup.and.burnin = 10000,
  obtain.standardized.cfa=TRUE,
  beta.o.prior.mean = 0,
  beta.o.prior.var = 100,
  psi.y.prior.alpha = 1,
  psi.y.prior.beta = 1,
  beta.c.prior.mean = 0,
  beta.c.prior.var = 100,
  n.iters.per.chain.total.structural = 51,
  save.summary.stats.from.MUPPET=TRUE,
  save.draws.from.MUPPET=FALSE,
  model.check=TRUE,
  save.post.pred.data=FALSE
)

# Examine summary statistics from model fitting, model-checking ------
MUPPET.CFA.outcome$summary.statistics.MUPPET
MUPPET.CFA.outcome$summary.statistics.PPMC





